# Generated by Django 4.1.4 on 2023-01-06 22:20

from django.db import migrations
import rest.fields


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0006_delete_eventmetadata'),
    ]

    operations = [
        migrations.AddField(
            model_name='event',
            name='metadata',
            field=rest.fields.JSONField(default=None, null=True),
        ),
    ]
