# Generated by Django 4.1.4 on 2023-08-22 19:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0008_incident_action_sent_incident_hostname_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='incident',
            name='reporter_ip',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=16, null=True),
        ),
    ]
