# Generated by Django 4.0.3 on 2022-04-04 23:52

from django.db import migrations, models
import django.db.models.deletion
import rest.models


class Migration(migrations.Migration):

    dependencies = [
        ('location', '0001_initial'),
        ('account', '0006_member_security_token'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('last_activity', models.DateTimeField(db_index=True)),
                ('signature', models.CharField(db_index=True, max_length=127)),
                ('ip', models.CharField(blank=True, db_index=True, max_length=127, null=True)),
                ('user_agent', models.TextField(blank=True, null=True)),
                ('browser', models.CharField(blank=True, default=None, max_length=64, null=True)),
                ('os', models.CharField(blank=True, default=None, max_length=64, null=True)),
                ('device', models.CharField(blank=True, default=None, max_length=64, null=True)),
                ('location', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='auth_sessions', to='location.geoip')),
                ('member', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='auth_sessions', to='account.member')),
            ],
            bases=(models.Model, rest.models.RestModel),
        ),
    ]
