# Generated by Django 4.1.4 on 2023-06-23 22:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('metrics', '0004_eodmetrics'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metrics',
            name='v1',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v10',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v11',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v12',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v13',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v14',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v2',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v3',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v4',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v5',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v6',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v7',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v8',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='metrics',
            name='v9',
            field=models.BigIntegerField(default=0),
        ),
    ]
